# fy:start ===>>>
from typing import List


class ImportGeneric_UsingSetter_PropertyMixin:
    @property
    def _import_generic(self) -> List[str]:
        return self.__import_generic

    @_import_generic.setter
    def _import_generic(self, import_generic: List[str]) -> None:
        self.__import_generic = import_generic


# fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from typing import List


flow abstract_method_imports_flow -> List[str]:
    property parsed_abstract_method_fy_py_file using setter
    property import_abc using constant(["import abc"])
    property import_generic using constant(["from typing import Generic"])
fy"""

from typing import List, Any

from fy_core.base.flow_base import FlowBase
from fy_library.domain.parsed_fy_py_file import ParsedAbstractMethodFyPyFile
from fy_library.mixins.property.imports_and_user_imports.import_abc.using_setter import (
    ImportAbc_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.imports_and_user_imports.import_generic.using_setter import (
    ImportGeneric_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.parsed_entity_fy_py_file.parsed_abstract_method_fy_py_file.using_setter import (
    ParsedAbstractMethodFyPyFile_UsingSetter_PropertyMixin,
)


# fy:start ===>>>
class AbstractMethodImportsFlow_Flow(
    # Property Mixins
    ParsedAbstractMethodFyPyFile_UsingSetter_PropertyMixin,
    ImportAbc_UsingSetter_PropertyMixin,
    ImportGeneric_UsingSetter_PropertyMixin,
    # Base
    FlowBase[List[str]],
):
    def __init__(
        self,
        *args: Any,
        parsed_abstract_method_fy_py_file: ParsedAbstractMethodFyPyFile,
        **kwargs: Any,
    ):
        self._parsed_abstract_method_fy_py_file = parsed_abstract_method_fy_py_file
        self._import_abc = ["import abc"]
        self._import_generic = ["from typing import Generic"]
        super().__init__(*args, **kwargs)

    def __call__(self) -> List[str]:
        # fy:end <<<===
        return (
            self._import_abc + self._import_generic
            if self._parsed_abstract_method_fy_py_file.generics_def != ""
            else self._import_abc
        )



# fy:start ===>>>
from pathlib import Path


class FolderToParse_UsingSetter_PropertyMixin:
    @property
    def _folder_to_parse(self) -> Path:
        return self.__folder_to_parse

    @_folder_to_parse.setter
    def _folder_to_parse(self, folder_to_parse: Path) -> None:
        self.__folder_to_parse = folder_to_parse


# fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from fy_library.domain.parsed_fy_py_file import FyPyFileParts


property fy_py_file_parts: FyPyFileParts using fy_file_to_parse_docstring:
    property fy_py_file_to_parse
fy"""

import abc
import re
from functools import cached_property
from typing import Final

from fy_library.constants import (
    FY_PY_FILE_SIGNATURE,
    FY_CODE_FILE_END_SIGNATURE,
    FY_START_MARKER,
    FY_END_MARKER,
)
from fy_library.domain.parsed_fy_py_file import FyPyFileParts
from fy_library.mixins.property.fy_py_file.fy_py_file_parts.abc_fy import (
    FyPyFileParts_PropertyMixin_ABC,
)
from fy_library.mixins.property.fy_py_file.fy_py_file_to_parse.abc_fy import (
    FyPyFileToParse_PropertyMixin_ABC,
)

_FY_CODE_REGEX: Final = re.compile(
    rf"^(?P<pre_fy_code>\s*#.*\n)*"
    rf"{FY_PY_FILE_SIGNATURE}"
    rf"(?P<fy_code>.*)"
    rf"{FY_CODE_FILE_END_SIGNATURE}",
    flags=re.DOTALL,
)
_FY_END_MARKER_REGEX: Final = re.compile(rf"\s*{FY_END_MARKER}\n")


# fy:start ===>>>
class FyPyFileParts_UsingFyFileToParseDocstring_PropertyMixin(
    # Property Mixins
    FyPyFileParts_PropertyMixin_ABC,
    FyPyFileToParse_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _fy_py_file_parts(self) -> FyPyFileParts:
        # fy:end <<<===
        with open(file=self._fy_py_file_to_parse, mode="r") as fy_py_file:
            content = fy_py_file.read()

        fy_code_regex_search = _FY_CODE_REGEX.search(content)
        pre_fy_code = fy_code_regex_search.group("pre_fy_code") or ""
        fy_code = fy_code_regex_search.group("fy_code")

        non_fy_code = content[
            len(
                f"{pre_fy_code}{FY_PY_FILE_SIGNATURE}{fy_code}{FY_CODE_FILE_END_SIGNATURE}\n"
            ) :  # fmt: skip
        ]

        pre_marker_file_content = non_fy_code.split(f"{FY_START_MARKER}\n")[0]
        post_marker_file_content = _FY_END_MARKER_REGEX.split(non_fy_code)[-1]

        fy_py_file_parts = FyPyFileParts(
            pre_fy_code=pre_fy_code,
            fy_code=fy_code,
            pre_marker_file_content=pre_marker_file_content,
            post_marker_file_content=post_marker_file_content,
        )

        return fy_py_file_parts



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
import abc
from enum import Enum

from pydantic import BaseModel, computed_field

from fy_library.constants import (
    GENERATED_PROPERTY_FILE_IMPLEMENTATION_NAMES,
    PROPERTY_SETTER_IMPLEMENTATION_NAME,
)
from fy_library.domain.entity_key import entity_key
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.domain.python_entity_name import PythonEntityName


class MixinModelKind(Enum):
    ABSTRACT_PROPERTY = "abstract_property"
    ABSTRACT_METHOD = "abstract_method"
    PROPERTY = "property"
    METHOD = "method"


class BaseMixinModel(BaseModel, abc.ABC):
    python_class_name: PythonEntityName
    kind: MixinModelKind

    @property
    @abc.abstractmethod
    def entity_key(self) -> tuple[ParsedFyPyFileKind, str]:
        raise NotImplementedError()


class AbstractMethodModel(BaseMixinModel):
    method_name: PythonEntityName
    generics_impl: str
    return_type: str

    @computed_field
    @property
    def entity_key(self) -> tuple[ParsedFyPyFileKind, str]:
        return ParsedFyPyFileKind(self.kind.value), self.method_name.snake_case


class MethodMixinModel(AbstractMethodModel):
    implementation_name: PythonEntityName

    @computed_field
    @property
    def entity_key(self) -> tuple[ParsedFyPyFileKind, str]:
        return entity_key(
            fy_py_kind=ParsedFyPyFileKind(self.kind.value),
            mixin_name__snake_case=self.method_name.snake_case,
            mixin_implementation_name__snake_case=self.implementation_name.snake_case,
        )


class AbstractPropertyModel(BaseMixinModel):
    property_name: PythonEntityName
    generics_impl: str
    property_type: str

    @computed_field
    @property
    def entity_key(self) -> tuple[ParsedFyPyFileKind, str]:
        return ParsedFyPyFileKind(self.kind.value), self.property_name.snake_case


class PropertyMixinModel(AbstractPropertyModel):
    implementation_name: PythonEntityName
    constant_value: str

    @computed_field
    @property
    def entity_key(self) -> tuple[ParsedFyPyFileKind, str]:
        fy_py_kind = (
            ParsedFyPyFileKind.PROPERTY_SETTER
            if self.implementation_name.snake_case
            in GENERATED_PROPERTY_FILE_IMPLEMENTATION_NAMES
            else ParsedFyPyFileKind.PROPERTY
        )
        return entity_key(
            fy_py_kind=fy_py_kind,
            mixin_name__snake_case=self.property_name.snake_case,
            mixin_implementation_name__snake_case=self.implementation_name.snake_case
            if fy_py_kind != ParsedFyPyFileKind.PROPERTY_SETTER
            else PROPERTY_SETTER_IMPLEMENTATION_NAME,
        )



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
flow generate_and_save_flow_compose_file__using_parsed_fy_py_file -> None:
    property parsed_fy_py_file using setter
    property mixin_import_map using setter
    property parsed_fy_py_files_map_by_key using setter
    property abstract_entities_ordering_index using setter
    property mixin_imports using parsed_fy_py_file
    property jinja2_template_file_name using parsed_fy_py_file
    property template_model using parsed_fy_py_file
    property flow_compose_file_content using parsed_fy_py_file
    property generated_flow_compose_code using jinja2_templates
    property flow_compose_file_name using parsed_fy_py_file
    method generate_and_save_flow_compose_code using flow_compose_file_content
fy"""

from typing import Any

from fy_core.base.flow_base import FlowBase

from fy_library.domain.parsed_fy_py_file import (
    ParsedFyPyFile,
)
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.mixins.method.generate_and_save_flow_compose_code.using_generated_flow_compose_code_fy import (
    GenerateAndSaveFlowComposeCode_UsingFlowComposeFileContent_MethodMixin,
)
from fy_library.mixins.property.flow_compose_file.flow_compose_file_name.using_parsed_fy_py_file_fy import (
    FlowComposeFileName_UsingParsedFyPyFile_PropertyMixin,
)
from fy_library.mixins.property.imports_and_user_imports.mixin_import_map.using_setter import (
    MixinImportMap_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.imports_and_user_imports.mixin_imports.using_parsed_fy_py_file_fy import (
    MixinImports_UsingParsedFyPyFile_PropertyMixin,
)
from fy_library.mixins.property.mro.ordered_abstract_entities.using_setter import (
    AbstractEntitiesOrderingIndex_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_file.using_setter import (
    ParsedFyPyFile_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files_map_by_key.using_setter import (
    ParsedFyPyFilesMapByKey_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.templates.generated_flow_compose_code.using_jinja2_templates_fy import (
    GeneratedFlowComposeCode_UsingJinja2Templates_PropertyMixin,
)
from fy_library.mixins.property.templates.jinja2_template_file_name.using_parsed_fy_py_file_fy import (
    Jinja2TemplateFileName_UsingParsedFyPyFile_PropertyMixin,
)
from fy_library.mixins.property.templates.template_model.using_parsed_fy_py_file_fy import (
    TemplateModel_UsingParsedFyPyFile_PropertyMixin,
)

from fy_library.mixins.property.flow_compose_file.flow_compose_file_content.using_parsed_fy_py_file_fy import (
    FlowComposeFileContent_UsingParsedFyPyFile_PropertyMixin,
)


# fy:start ===>>>
class GenerateAndSaveFlowComposeFile_UsingParsedFyPyFile_Flow(
    # Property Mixins
    ParsedFyPyFile_UsingSetter_PropertyMixin,
    MixinImportMap_UsingSetter_PropertyMixin,
    ParsedFyPyFilesMapByKey_UsingSetter_PropertyMixin,
    AbstractEntitiesOrderingIndex_UsingSetter_PropertyMixin,
    MixinImports_UsingParsedFyPyFile_PropertyMixin,
    Jinja2TemplateFileName_UsingParsedFyPyFile_PropertyMixin,
    TemplateModel_UsingParsedFyPyFile_PropertyMixin,
    FlowComposeFileContent_UsingParsedFyPyFile_PropertyMixin,
    GeneratedFlowComposeCode_UsingJinja2Templates_PropertyMixin,
    FlowComposeFileName_UsingParsedFyPyFile_PropertyMixin,
    # Method Mixins
    GenerateAndSaveFlowComposeCode_UsingFlowComposeFileContent_MethodMixin,
    # Base
    FlowBase[None],
):
    def __init__(
        self,
        *args: Any,
        parsed_fy_py_file: ParsedFyPyFile,
        mixin_import_map: dict[tuple[ParsedFyPyFileKind, str], str],
        parsed_fy_py_files_map_by_key: dict[
            tuple[ParsedFyPyFileKind, str], ParsedFyPyFile
        ],
        abstract_entities_ordering_index: dict[tuple[ParsedFyPyFileKind, str], int],
        **kwargs: Any,
    ):
        self._parsed_fy_py_file = parsed_fy_py_file
        self._mixin_import_map = mixin_import_map
        self._parsed_fy_py_files_map_by_key = parsed_fy_py_files_map_by_key
        self._abstract_entities_ordering_index = abstract_entities_ordering_index
        super().__init__(*args, **kwargs)

    def __call__(self) -> None:
        # fy:end <<<===
        self._generate_and_save_flow_compose_code()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from fy_library.domain.fy_py_template_models import BaseTemplateModel


flow create_template_model_using_parsed_fy_py_file -> BaseTemplateModel:
    property parsed_fy_py_file using setter
    property parsed_fy_py_files_map_by_key using setter
    property abstract_entities_ordering_index using setter
fy"""

from typing import Any

from fy_core.base.flow_base import FlowBase

from fy_library.domain.fy_py_template_models import (
    BaseTemplateModel,
)
from fy_library.domain.parsed_fy_py_file import ParsedFyPyFile
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.flows.create_template_model_using_parsed_fy_py_file.abstract_method_template_model_fy import (
    CreateAbstractMethodTemplateModel_UsingParsedFyPyFile_Flow,
)
from fy_library.flows.create_template_model_using_parsed_fy_py_file.abstract_property_template_model_fy import (
    CreateAbstractPropertyTemplateModel_UsingParsedFyPyFile_Flow,
)
from fy_library.flows.create_template_model_using_parsed_fy_py_file.base_flow_template_model_fy import (
    CreateBaseFlowTemplateModel_UsingParsedFyPyFileAndPropertySettersTemplateModels_Flow,
)
from fy_library.flows.create_template_model_using_parsed_fy_py_file.flow_template_model_fy import (
    CreateFlowTemplateModel_UsingParsedFyPyFileAndPropertySettersTemplateModels_Flow,
)
from fy_library.flows.create_template_model_using_parsed_fy_py_file.method_template_model_fy import (
    CreateMethodTemplateModel_UsingParsedFyPyFile_Flow,
)
from fy_library.flows.create_template_model_using_parsed_fy_py_file.property_template_model_fy import (
    CreatePropertyTemplateModel_UsingParsedFyPyFile_Flow,
)
from fy_library.mixins.property.mro.ordered_abstract_entities.using_setter import (
    AbstractEntitiesOrderingIndex_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_file.using_setter import (
    ParsedFyPyFile_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files_map_by_key.using_setter import (
    ParsedFyPyFilesMapByKey_UsingSetter_PropertyMixin,
)


# fy:start ===>>>
class CreateTemplateModelUsingParsedFyPyFile_Flow(
    # Property Mixins
    ParsedFyPyFile_UsingSetter_PropertyMixin,
    ParsedFyPyFilesMapByKey_UsingSetter_PropertyMixin,
    AbstractEntitiesOrderingIndex_UsingSetter_PropertyMixin,
    # Base
    FlowBase[BaseTemplateModel],
):
    def __init__(
        self,
        *args: Any,
        parsed_fy_py_file: ParsedFyPyFile,
        parsed_fy_py_files_map_by_key: dict[
            tuple[ParsedFyPyFileKind, str], ParsedFyPyFile
        ],
        abstract_entities_ordering_index: dict[tuple[ParsedFyPyFileKind, str], int],
        **kwargs: Any,
    ):
        self._parsed_fy_py_file = parsed_fy_py_file
        self._parsed_fy_py_files_map_by_key = parsed_fy_py_files_map_by_key
        self._abstract_entities_ordering_index = abstract_entities_ordering_index
        super().__init__(*args, **kwargs)

    def __call__(self) -> BaseTemplateModel:
        # fy:end <<<===
        match self._parsed_fy_py_file.file_type:
            case ParsedFyPyFileKind.FLOW:
                return CreateFlowTemplateModel_UsingParsedFyPyFileAndPropertySettersTemplateModels_Flow(
                    parsed_fy_py_file=self._parsed_fy_py_file,
                    parsed_fy_py_files_map_by_key=self._parsed_fy_py_files_map_by_key,
                )()
            case ParsedFyPyFileKind.BASE_FLOW:
                return CreateBaseFlowTemplateModel_UsingParsedFyPyFileAndPropertySettersTemplateModels_Flow(
                    parsed_fy_py_file=self._parsed_fy_py_file,
                    parsed_fy_py_files_map_by_key=self._parsed_fy_py_files_map_by_key,
                    abstract_entities_ordering_index=self._abstract_entities_ordering_index,
                )()
            case ParsedFyPyFileKind.ABSTRACT_PROPERTY:
                return CreateAbstractPropertyTemplateModel_UsingParsedFyPyFile_Flow(
                    parsed_fy_py_file=self._parsed_fy_py_file,
                )()
            case ParsedFyPyFileKind.ABSTRACT_METHOD:
                return CreateAbstractMethodTemplateModel_UsingParsedFyPyFile_Flow(
                    parsed_fy_py_file=self._parsed_fy_py_file,
                )()
            case ParsedFyPyFileKind.PROPERTY:
                return CreatePropertyTemplateModel_UsingParsedFyPyFile_Flow(
                    parsed_fy_py_file=self._parsed_fy_py_file,
                    abstract_entities_ordering_index=self._abstract_entities_ordering_index,
                    parsed_fy_py_files_map_by_key=self._parsed_fy_py_files_map_by_key,
                )()
            case ParsedFyPyFileKind.METHOD:
                return CreateMethodTemplateModel_UsingParsedFyPyFile_Flow(
                    parsed_fy_py_file=self._parsed_fy_py_file,
                    abstract_entities_ordering_index=self._abstract_entities_ordering_index,
                    parsed_fy_py_files_map_by_key=self._parsed_fy_py_files_map_by_key,
                )()

        raise NotImplementedError(
            f"No template model found for file type '{self._parsed_fy_py_file.file_type}' in '{self._parsed_fy_py_file.file_path}'"
        )



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
property jinja2_template_file_name: str using property_setter_constant:
fy"""

import abc
from functools import cached_property

from fy_library.mixins.property.templates.jinja2_template_file_name.abc_fy import (
    Jinja2TemplateFileName_PropertyMixin_ABC,
)


# fy:start ===>>>
class Jinja2TemplateFileName_UsingPropertySetterConstant_PropertyMixin(
    # Property Mixins
    Jinja2TemplateFileName_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _jinja2_template_file_name(self) -> str:
        # fy:end <<<===
        return "property_setter.jinja2"



# fy:start ===>>>
from fy_library.domain.parsed_fy_py_file import ParsedFyPyFile
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind


class ParsedFyPyFilesMapByKey_UsingSetter_PropertyMixin:
    @property
    def _parsed_fy_py_files_map_by_key(
        self,
    ) -> dict[tuple[ParsedFyPyFileKind, str], ParsedFyPyFile]:
        return self.__parsed_fy_py_files_map_by_key

    @_parsed_fy_py_files_map_by_key.setter
    def _parsed_fy_py_files_map_by_key(
        self,
        parsed_fy_py_files_map_by_key: dict[
            tuple[ParsedFyPyFileKind, str], ParsedFyPyFile
        ],
    ) -> None:
        self.__parsed_fy_py_files_map_by_key = parsed_fy_py_files_map_by_key


# fy:end <<<===



# fy:start ===>>>
from typing import List


class ImportAbc_UsingSetter_PropertyMixin:
    @property
    def _import_abc(self) -> List[str]:
        return self.__import_abc

    @_import_abc.setter
    def _import_abc(self, import_abc: List[str]) -> None:
        self.__import_abc = import_abc


# fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from fy_library.domain.parsed_fy_py_file import ParsedFyPyFile
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind


property parsed_fy_py_files_map_by_key: dict[tuple[ParsedFyPyFileKind, str], ParsedFyPyFile]
fy"""

import abc

from fy_library.domain.parsed_fy_py_file import ParsedFyPyFile
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind


# fy:start ===>>>
class ParsedFyPyFilesMapByKey_PropertyMixin_ABC(abc.ABC):
    @property
    @abc.abstractmethod
    def _parsed_fy_py_files_map_by_key(
        self,
    ) -> dict[tuple[ParsedFyPyFileKind, str], ParsedFyPyFile]:
        raise NotImplementedError()
        # fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
property abstract_entities_ordering_index: dict[tuple[ParsedFyPyFileKind, str], int] using parsed_fy_py_files_map_by_key:
    property parsed_fy_py_files_map_by_key
fy"""

import abc
from functools import cached_property
from typing import cast

from fy_library.domain.parsed_fy_py_file import (
    ParsedAbstractPropertyFyPyFile,
    ParsedAbstractMethodFyPyFile,
)
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.mixins.property.mro.ordered_abstract_entities.abc_fy import (
    AbstractEntitiesOrderingIndex_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files_map_by_key.abc_fy import (
    ParsedFyPyFilesMapByKey_PropertyMixin_ABC,
)


# fy:start ===>>>
class AbstractEntitiesOrderingIndex_UsingParsedFyPyFilesMapByKey_PropertyMixin(
    # Property Mixins
    AbstractEntitiesOrderingIndex_PropertyMixin_ABC,
    ParsedFyPyFilesMapByKey_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _abstract_entities_ordering_index(
        self,
    ) -> dict[tuple[ParsedFyPyFileKind, str], int]:
        # fy:end <<<===
        def sorting_mixin_key(
            mixin_key: tuple[ParsedFyPyFileKind, str],
        ) -> tuple[int, tuple[ParsedFyPyFileKind, str]]:
            parsed_fy_py_file = self._parsed_fy_py_files_map_by_key[mixin_key]
            prefix_index: int
            match parsed_fy_py_file.file_type:
                case ParsedFyPyFileKind.ABSTRACT_PROPERTY:
                    abstract_property_parsed_fy_py_file = cast(
                        ParsedAbstractPropertyFyPyFile, parsed_fy_py_file
                    )
                    prefix_index = (
                        1
                        if len(abstract_property_parsed_fy_py_file.generics_def) > 0
                        else 0
                    )
                case ParsedFyPyFileKind.ABSTRACT_METHOD:
                    abstract_method_parsed_fy_py_file = cast(
                        ParsedAbstractMethodFyPyFile, parsed_fy_py_file
                    )
                    prefix_index = (
                        1
                        if len(abstract_method_parsed_fy_py_file.generics_def) > 0
                        else 0
                    )
                case _:
                    prefix_index = 0

            return prefix_index, mixin_key

        abstract_entities_ordering_index = {
            entity_key: entity_num
            for entity_num, entity_key in enumerate(
                sorted(self._parsed_fy_py_files_map_by_key, key=sorting_mixin_key)
            )
            if self._parsed_fy_py_files_map_by_key[entity_key].file_type
            in {
                ParsedFyPyFileKind.ABSTRACT_METHOD,
                ParsedFyPyFileKind.ABSTRACT_PROPERTY,
            }
        }
        return abstract_entities_ordering_index



# fy:start ===>>>
from fy_library.domain.parsed_fy_py_file import ParsedAbstractMethodFyPyFile


class ParsedAbstractMethodFyPyFile_UsingSetter_PropertyMixin:
    @property
    def _parsed_abstract_method_fy_py_file(self) -> ParsedAbstractMethodFyPyFile:
        return self.__parsed_abstract_method_fy_py_file

    @_parsed_abstract_method_fy_py_file.setter
    def _parsed_abstract_method_fy_py_file(
        self, parsed_abstract_method_fy_py_file: ParsedAbstractMethodFyPyFile
    ) -> None:
        self.__parsed_abstract_method_fy_py_file = parsed_abstract_method_fy_py_file


# fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
import abc
from typing import List

from pydantic import BaseModel

from fy_library.domain.mixin_models import (
    MethodMixinModel,
    AbstractMethodModel,
    AbstractPropertyModel,
    PropertyMixinModel,
)
from fy_library.domain.parsed_fy_py_file import ParsedAbstractPropertyFyPyFile
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.domain.python_entity_name import PythonEntityName


class BaseTemplateModel(BaseModel, abc.ABC):
    python_class_name: PythonEntityName


class TemporaryBaseTemplateModel(BaseTemplateModel):
    entity_key_value: tuple[ParsedFyPyFileKind, str]


class MethodTemplateModel(BaseTemplateModel):
    method_name: PythonEntityName
    abstract_property_non_generic_mixins: List[AbstractPropertyModel]
    abstract_property_generic_mixins: List[AbstractPropertyModel]
    abstract_method_non_generic_mixins: List[AbstractMethodModel]
    abstract_method_generic_mixins: List[AbstractMethodModel]
    generics_def: str
    arguments: str | None
    implementation_name: PythonEntityName
    return_type: str


class AbstractMethodTemplateModel(BaseTemplateModel):
    abstract_method_name: PythonEntityName
    generics_def: str
    arguments: str | None
    return_type: str


class AbstractPropertyTemplateModel(BaseTemplateModel):
    abstract_property_name: PythonEntityName
    generics_def: str
    property_type: str


class FlowTemplateModel(BaseTemplateModel):
    flow_name: PythonEntityName
    generics_def: str
    declared_base_flow: PythonEntityName | None
    declared_base_flow_generics_def: str
    return_type: str
    properties: list[PropertyMixinModel]
    properties_without_setters: list[PropertyMixinModel]
    methods: list[MethodMixinModel]
    property_setters: list[ParsedAbstractPropertyFyPyFile]
    property_constant_setters: list[PropertyMixinModel]
    property_mixins_in_use: list[PropertyMixinModel]
    method_mixins_in_use: list[MethodMixinModel]


class BaseFlowTemplateModel(BaseTemplateModel):
    base_flow_name: PythonEntityName
    generics_def: str
    declared_base_flow_python_class_name: PythonEntityName | None
    declared_base_flow_generics_def: str
    return_type: str
    properties: List[PropertyMixinModel]
    methods: List[MethodMixinModel]
    abstract_property_mixins: List[AbstractPropertyModel]
    abstract_method_mixins: List[AbstractMethodModel]
    property_setters: List[ParsedAbstractPropertyFyPyFile]
    callable_annotation: bool


class PropertyTemplateModel(BaseTemplateModel):
    property_name: PythonEntityName
    implementation_name: PythonEntityName
    abstract_property_mixins: List[AbstractPropertyModel]
    generics_def: str
    property_type: str


class PropertySetterTemplateModel(BaseTemplateModel):
    property_name: PythonEntityName
    property_type: str
    generics_def: str



# fy:start ===>>>
from fy_library.domain.parsed_fy_py_file import ParsedFyPyFile


class ParsedFyPyFile_UsingSetter_PropertyMixin:
    @property
    def _parsed_fy_py_file(self) -> ParsedFyPyFile:
        return self.__parsed_fy_py_file

    @_parsed_fy_py_file.setter
    def _parsed_fy_py_file(self, parsed_fy_py_file: ParsedFyPyFile) -> None:
        self.__parsed_fy_py_file = parsed_fy_py_file


# fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from typing import List
from fy_library.domain.mixin_models import BaseMixinModel


property mixin_models: List[BaseMixinModel] using mixin_lines:
    property fy_py_file_to_parse
    property mixin_lines
fy"""

import abc
from functools import cached_property
from typing import List

from fy_library.domain.mixin_models import BaseMixinModel
from fy_library.flows.mixin_model.mixin_line_to_mixin_model_fy import (
    MixinLineToMixinModel_Flow,
)
from fy_library.mixins.property.fy_py_file.fy_py_file_to_parse.abc_fy import (
    FyPyFileToParse_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsing_mixin_model.mixin_lines.abc_fy import (
    MixinLines_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsing_mixin_model.mixin_models.abc_fy import (
    MixinModels_PropertyMixin_ABC,
)


# fy:start ===>>>
class MixinModels_UsingMixinLines_PropertyMixin(
    # Property Mixins
    FyPyFileToParse_PropertyMixin_ABC,
    MixinLines_PropertyMixin_ABC,
    MixinModels_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _mixin_models(self) -> List[BaseMixinModel]:
        # fy:end <<<===
        return [
            MixinLineToMixinModel_Flow(
                mixin_line=mixin_line, fy_py_file_to_parse=self._fy_py_file_to_parse
            )()
            for mixin_line in self._mixin_lines
            if mixin_line != ""
        ]



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from fy_library.domain.mixin_models import AbstractPropertyModel


property optional_abstract_property_mixin_model: AbstractPropertyModel | None using mixin_line:
    property mixin_line
fy"""

import abc
import re
from functools import cached_property
from typing import Final

from fy_library.constants import (
    FY_ENTITY_REGEX_STRING,
    PYTHON_MULTI_ENTITY_REGEX_STRING,
)
from fy_library.domain.mixin_models import MixinModelKind, AbstractPropertyModel
from fy_library.domain.python_entity_name import PythonEntityName
from fy_library.mixins.property.optional_mixin_models.optional_abstract_property_mixin_model.abc_fy import (
    OptionalAbstractPropertyMixinModel_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsing_mixin_model.mixin_line.abc_fy import (
    MixinLine_PropertyMixin_ABC,
)

_ABSTRACT_PROPERTY_MIXIN_REGEX: Final = re.compile(
    rf"^\s+property\s+(?P<abstract_property_name>{FY_ENTITY_REGEX_STRING})(?:\[(?P<generics_impl>{PYTHON_MULTI_ENTITY_REGEX_STRING})])?$"
)


# fy:start ===>>>
class OptionalAbstractPropertyMixinModel_UsingMixinLine_PropertyMixin(
    # Property Mixins
    MixinLine_PropertyMixin_ABC,
    OptionalAbstractPropertyMixinModel_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _optional_abstract_property_mixin_model(self) -> AbstractPropertyModel | None:
        # fy:end <<<===
        declared_abstract_property_mixin = _ABSTRACT_PROPERTY_MIXIN_REGEX.search(
            self._mixin_line
        )
        if declared_abstract_property_mixin is None:
            return None
        property_name = PythonEntityName.from_snake_case(
            declared_abstract_property_mixin.group("abstract_property_name")
        )

        return AbstractPropertyModel(
            python_class_name=PythonEntityName.from_pascal_case(
                f"{ property_name.pascal_case }_PropertyMixin_ABC"
            ),
            kind=MixinModelKind.ABSTRACT_PROPERTY,
            property_name=property_name,
            generics_impl=declared_abstract_property_mixin.group("generics_impl") or "",
            property_type="",
        )



# fy:start ===>>>
class PostMarkerFileContent_UsingSetter_PropertyMixin:
    @property
    def _post_marker_file_content(self) -> str:
        return self.__post_marker_file_content

    @_post_marker_file_content.setter
    def _post_marker_file_content(self, post_marker_file_content: str) -> None:
        self.__post_marker_file_content = post_marker_file_content


# fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from typing import List
from fy_library.domain.mixin_models import BaseMixinModel


property abstract_mixins: List[BaseMixinModel] using parsed_property_fy_py_file:
    property parsed_fy_py_file
fy"""

import abc
from functools import cached_property
from typing import List, cast

from fy_library.domain.mixin_models import BaseMixinModel
from fy_library.domain.parsed_fy_py_file import (
    ParsedPropertyFyPyFile,
)
from fy_library.mixins.property.entity_mixins.abstract_mixins.abc_fy import (
    AbstractMixins_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_file.abc_fy import (
    ParsedFyPyFile_PropertyMixin_ABC,
)


# fy:start ===>>>
class AbstractMixins_UsingParsedPropertyFyPyFile_PropertyMixin(
    # Property Mixins
    AbstractMixins_PropertyMixin_ABC,
    ParsedFyPyFile_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _abstract_mixins(self) -> List[BaseMixinModel]:
        # fy:end <<<===
        parsed_property_fy_py_file = self._parsed_fy_py_file
        assert isinstance(parsed_property_fy_py_file, ParsedPropertyFyPyFile)

        return cast(
            List[BaseMixinModel], parsed_property_fy_py_file.abstract_property_mixins
        )



# fy:start ===>>>
from typing import List
from fy_library.domain.mixin_models import PropertyMixinModel


class PropertyMixins_UsingSetter_PropertyMixin:
    @property
    def _property_mixins(self) -> List[PropertyMixinModel]:
        return self.__property_mixins

    @_property_mixins.setter
    def _property_mixins(self, property_mixins: List[PropertyMixinModel]) -> None:
        self.__property_mixins = property_mixins


# fy:end <<<===



# fy:start ===>>>
from fy_library.domain.parsed_fy_py_file import ParsedPropertyFyPyFile


class ParsedPropertyFyPyFile_UsingSetter_PropertyMixin:
    @property
    def _parsed_property_fy_py_file(self) -> ParsedPropertyFyPyFile:
        return self.__parsed_property_fy_py_file

    @_parsed_property_fy_py_file.setter
    def _parsed_property_fy_py_file(
        self, parsed_property_fy_py_file: ParsedPropertyFyPyFile
    ) -> None:
        self.__parsed_property_fy_py_file = parsed_property_fy_py_file


# fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from fy_library.domain.parsed_fy_py_file import ParsedAbstractMethodFyPyFile


property parsed_abstract_method_fy_py_file: ParsedAbstractMethodFyPyFile using  parsed_fy_py_file:
    property pre_fy_code
    property fy_code
    property pre_marker_file_content
    property post_marker_file_content
    property fy_py_file_to_parse
    property abstract_method_file_split
fy"""

import abc
from functools import cached_property

from fy_library.domain.fy_py_template_models import TemporaryBaseTemplateModel
from fy_library.domain.parsed_fy_py_file import ParsedAbstractMethodFyPyFile
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.domain.python_entity_name import PythonEntityName
from fy_library.mixins.property.fy_file.fy_code.abc_fy import FyCode_PropertyMixin_ABC
from fy_library.mixins.property.fy_file.post_marker_file_content.abc_fy import (
    PostMarkerFileContent_PropertyMixin_ABC,
)
from fy_library.mixins.property.fy_file.pre_fy_code.abc_fy import (
    PreFyCode_PropertyMixin_ABC,
)
from fy_library.mixins.property.fy_file.pre_marker_file_content.abc_fy import (
    PreMarkerFileContent_PropertyMixin_ABC,
)
from fy_library.mixins.property.fy_py_file.fy_py_file_to_parse.abc_fy import (
    FyPyFileToParse_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_entity_fy_py_file.parsed_abstract_method_fy_py_file.abc_fy import (
    ParsedAbstractMethodFyPyFile_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_file_split.abstract_method_file_split.abc_fy import (
    AbstractMethodFileSplit_PropertyMixin_ABC,
)


# fy:start ===>>>
class ParsedAbstractMethodFyPyFile_UsingParsedFyPyFile_PropertyMixin(
    # Property Mixins
    AbstractMethodFileSplit_PropertyMixin_ABC,
    FyCode_PropertyMixin_ABC,
    FyPyFileToParse_PropertyMixin_ABC,
    ParsedAbstractMethodFyPyFile_PropertyMixin_ABC,
    PostMarkerFileContent_PropertyMixin_ABC,
    PreFyCode_PropertyMixin_ABC,
    PreMarkerFileContent_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _parsed_abstract_method_fy_py_file(self) -> ParsedAbstractMethodFyPyFile:
        # fy:end <<<===
        abstract_method_name = PythonEntityName.from_snake_case(
            self._abstract_method_file_split.abstract_method_name
        )

        parsed_fy_py_file = ParsedAbstractMethodFyPyFile(
            pre_fy_code=self._pre_fy_code,
            fy_code=self._fy_code,
            pre_marker_file_content=self._pre_marker_file_content,
            post_marker_file_content=self._post_marker_file_content,
            file_path=self._fy_py_file_to_parse,
            user_imports=self._abstract_method_file_split.user_imports,
            abstract_method_name=abstract_method_name,
            generics_def=self._abstract_method_file_split.generics_def,
            arguments=self._abstract_method_file_split.arguments,
            return_type=self._abstract_method_file_split.return_type,
            python_class_name=PythonEntityName.from_pascal_case(
                f"{abstract_method_name.pascal_case}_MethodMixin_ABC"
            ),
            template_model=TemporaryBaseTemplateModel(
                python_class_name=PythonEntityName.from_pascal_case(
                    f"{abstract_method_name.pascal_case}_MethodMixin_ABC"
                ),
                entity_key_value=(
                    ParsedFyPyFileKind.ABSTRACT_METHOD,
                    abstract_method_name.snake_case,
                ),
            ),
        )

        return parsed_fy_py_file



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from pathlib import Path

property flow_compose_file_name: Path using parsed_fy_py_file:
    property parsed_fy_py_file
fy"""

import abc
from functools import cached_property

from fy_library.constants import FY_PY_FILE_EXTENSION
from fy_library.mixins.property.flow_compose_file.flow_compose_file_name.abc_fy import (
    FlowComposeFileName_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_file.abc_fy import (
    ParsedFyPyFile_PropertyMixin_ABC,
)

from pathlib import Path


# fy:start ===>>>
class FlowComposeFileName_UsingParsedFyPyFile_PropertyMixin(
    # Property Mixins
    FlowComposeFileName_PropertyMixin_ABC,
    ParsedFyPyFile_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _flow_compose_file_name(self) -> Path:
        # fy:end <<<===
        assert self._parsed_fy_py_file.file_path.name.endswith(FY_PY_FILE_EXTENSION)
        fc_file_name = f"{self._parsed_fy_py_file.file_path.name[: -len(FY_PY_FILE_EXTENSION)]}_fc.py"
        return self._parsed_fy_py_file.file_path.parent / fc_file_name



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
property base_flow_file_split: BaseFlowFileSplitModel
fy"""

import abc
from typing import List

from pydantic import BaseModel

from fy_library.domain.annotation_object import Annotation


class BaseFlowFileSplitModel(BaseModel):
    user_imports: str
    annotations: List[Annotation]
    base_flow_name: str
    generics_def: str
    declared_base_flow: str
    declared_base_flow_generics_def: str
    return_type: str
    mixins: str


# fy:start ===>>>
class BaseFlowFileSplit_PropertyMixin_ABC(abc.ABC):
    @property
    @abc.abstractmethod
    def _base_flow_file_split(self) -> BaseFlowFileSplitModel:
        raise NotImplementedError()
        # fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
property flow_file_split: FlowFileSplitModel using flow_regex:
    property fy_py_file_to_parse
    property fy_code
fy"""

import abc
import re
from functools import cached_property
from typing import Final

from fy_library.constants import (
    FY_ENTITY_REGEX_STRING,
    PYTHON_MULTI_ENTITY_REGEX_STRING,
)
from fy_library.mixins.property.fy_file.fy_code.abc_fy import FyCode_PropertyMixin_ABC
from fy_library.mixins.property.fy_py_file.fy_py_file_to_parse.abc_fy import (
    FyPyFileToParse_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_file_split.flow_file_split.abc_fy import (
    FlowFileSplit_PropertyMixin_ABC,
    FlowFileSplitModel,
)

_FLOW_STRING_SPLIT_REGEX: Final = re.compile(
    rf"flow\s+(?P<flow_name>{FY_ENTITY_REGEX_STRING})\s*"
    rf"(?:\[(?P<generics_def>{PYTHON_MULTI_ENTITY_REGEX_STRING})])?\s*"
    rf"(?:\((?P<declared_base_flow>{FY_ENTITY_REGEX_STRING})(?:\[(?P<declared_base_flow_generics_def>{PYTHON_MULTI_ENTITY_REGEX_STRING})])?\))?"
    rf"(?:\s+->\s+(?P<return_type>{PYTHON_MULTI_ENTITY_REGEX_STRING}))?:\s*\n"
)


# fy:start ===>>>
class FlowFileSplit_UsingFlowRegex_PropertyMixin(
    # Property Mixins
    FlowFileSplit_PropertyMixin_ABC,
    FyCode_PropertyMixin_ABC,
    FyPyFileToParse_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _flow_file_split(self) -> FlowFileSplitModel:
        # fy:end <<<===
        flow_file_split = _FLOW_STRING_SPLIT_REGEX.split(self._fy_code)

        assert (
            (len(flow_file_split)) == 7
        ), f"Flow file split length {len(flow_file_split)} is invalid in {self._fy_py_file_to_parse}"

        flow_file_split_model = FlowFileSplitModel(
            user_imports=flow_file_split[0],
            flow_name=flow_file_split[1],
            generics_def=flow_file_split[2] or "",
            declared_base_flow=flow_file_split[3] or "",
            declared_base_flow_generics_def=flow_file_split[4] or "",
            return_type=flow_file_split[5] or "",
            mixins=flow_file_split[6],
        )

        return flow_file_split_model



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from typing import List
from fy_library.domain.parsed_fy_py_file import ParsedFyPyFile


property parsed_fy_py_files: List[ParsedFyPyFile] using fy_py_parser:
    property fy_py_files_to_parse
fy"""

import abc
from functools import cached_property
from typing import List

from fy_library.domain.parsed_fy_py_file import ParsedFyPyFile
from fy_library.flows.parse_fy_py_file_fy import ParseFyPyFile_Flow
from fy_library.mixins.property.fy_py_file.fy_py_files_to_parse.abc_fy import (
    FyPyFilesToParse_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files.abc_fy import (
    ParsedFyPyFiles_PropertyMixin_ABC,
)


# fy:start ===>>>
class ParsedFyPyFiles_UsingFyPyParser_PropertyMixin(
    # Property Mixins
    FyPyFilesToParse_PropertyMixin_ABC,
    ParsedFyPyFiles_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _parsed_fy_py_files(self) -> List[ParsedFyPyFile]:
        # fy:end <<<===
        return [
            ParseFyPyFile_Flow(fy_py_file_to_parse=fy_py_file)()
            for fy_py_file in self._fy_py_files_to_parse
        ]



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
property included_mixins: IncludedMixinsModel
fy"""

import abc
from typing import List

from pydantic import BaseModel

from fy_library.domain.mixin_models import (
    MethodMixinModel,
    AbstractMethodModel,
    AbstractPropertyModel,
    PropertyMixinModel,
)


class IncludedMixinsModel(BaseModel):
    abstract_method_mixins: List[AbstractMethodModel]
    abstract_property_mixins: List[AbstractPropertyModel]
    method_mixins: List[MethodMixinModel]
    property_mixins: List[PropertyMixinModel]


# fy:start ===>>>
class IncludedMixins_PropertyMixin_ABC(abc.ABC):
    @property
    @abc.abstractmethod
    def _included_mixins(self) -> IncludedMixinsModel:
        raise NotImplementedError()
        # fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
method generate_and_save_fy_py_code -> None using parsed_fy_py_file__and__fy_py_file_content:
    property parsed_fy_py_file
    property fy_py_file_content
fy"""

import abc

from fy_library.mixins.property.fy_py_file.fy_py_file_content.abc_fy import (
    FyPyFileContent_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_file.abc_fy import (
    ParsedFyPyFile_PropertyMixin_ABC,
)


# fy:start ===>>>
class GenerateAndSaveFyPyCode_UsingParsedFyPyFile_And_FyPyFileContent_MethodMixin(
    # Property Mixins
    FyPyFileContent_PropertyMixin_ABC,
    ParsedFyPyFile_PropertyMixin_ABC,
    abc.ABC,
):
    def _generate_and_save_fy_py_code(self) -> None:
        # fy:end <<<===
        fy_py_file_content = self._fy_py_file_content
        with open(
            file=self._parsed_fy_py_file.file_path, mode="w", encoding="UTF-8"
        ) as setter_file:
            setter_file.write(fy_py_file_content)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
flow mixin_line_to_mixin_model -> BaseMixinModel:
    property mixin_line using setter
    property fy_py_file_to_parse using setter
    property optional_property_mixin_model using mixin_line
    property optional_method_mixin_model using mixin_line
    property optional_abstract_property_mixin_model using mixin_line
    property optional_abstract_method_mixin_model using mixin_line
fy"""

from pathlib import Path
from typing import Any

from fy_core.base.flow_base import FlowBase
from fy_library.domain.mixin_models import BaseMixinModel
from fy_library.mixins.property.fy_py_file.fy_py_file_to_parse.using_setter import (
    FyPyFileToParse_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.optional_mixin_models.optional_abstract_method_mixin_model.using_mixin_line_fy import (
    OptionalAbstractMethodMixinModel_UsingMixinLine_PropertyMixin,
)
from fy_library.mixins.property.optional_mixin_models.optional_abstract_property_mixin_model.using_mixin_line_fy import (
    OptionalAbstractPropertyMixinModel_UsingMixinLine_PropertyMixin,
)
from fy_library.mixins.property.optional_mixin_models.optional_method_mixin_model.using_mixin_line_fy import (
    OptionalMethodMixinModel_UsingMixinLine_PropertyMixin,
)
from fy_library.mixins.property.optional_mixin_models.optional_property_mixin_model.using_mixin_line_fy import (
    OptionalPropertyMixinModel_UsingMixinLine_PropertyMixin,
)
from fy_library.mixins.property.parsing_mixin_model.mixin_line.using_setter import (
    MixinLine_UsingSetter_PropertyMixin,
)


# fy:start ===>>>
class MixinLineToMixinModel_Flow(
    # Property Mixins
    MixinLine_UsingSetter_PropertyMixin,
    FyPyFileToParse_UsingSetter_PropertyMixin,
    OptionalPropertyMixinModel_UsingMixinLine_PropertyMixin,
    OptionalMethodMixinModel_UsingMixinLine_PropertyMixin,
    OptionalAbstractPropertyMixinModel_UsingMixinLine_PropertyMixin,
    OptionalAbstractMethodMixinModel_UsingMixinLine_PropertyMixin,
    # Base
    FlowBase[BaseMixinModel],
):
    def __init__(
        self,
        *args: Any,
        mixin_line: str,
        fy_py_file_to_parse: Path,
        **kwargs: Any,
    ):
        self._mixin_line = mixin_line
        self._fy_py_file_to_parse = fy_py_file_to_parse
        super().__init__(*args, **kwargs)

    def __call__(self) -> BaseMixinModel:
        # fy:end <<<===
        mixin_model = (
            self._optional_property_mixin_model
            or self._optional_method_mixin_model
            or self._optional_abstract_property_mixin_model
            or self._optional_abstract_method_mixin_model
        )

        if mixin_model is None:
            raise ValueError(
                f"Line {self._mixin_line} in {self._fy_py_file_to_parse} is invalid."
            )
        return mixin_model



# fy:start ===>>>
class FyCode_UsingSetter_PropertyMixin:
    @property
    def _fy_code(self) -> str:
        return self.__fy_code

    @_fy_code.setter
    def _fy_code(self, fy_code: str) -> None:
        self.__fy_code = fy_code


# fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from fy_library.domain.fy_py_template_models import BaseTemplateModel


property template_model: BaseTemplateModel using parsed_fy_py_file:
    property parsed_fy_py_file
    property parsed_fy_py_files_map_by_key
    property abstract_entities_ordering_index
fy"""

import abc
from functools import cached_property

from fy_library.domain.fy_py_template_models import (
    BaseTemplateModel,
)
from fy_library.flows.create_template_model_using_parsed_fy_py_file.main_fy import (
    CreateTemplateModelUsingParsedFyPyFile_Flow,
)
from fy_library.mixins.property.mro.ordered_abstract_entities.abc_fy import (
    AbstractEntitiesOrderingIndex_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_file.abc_fy import (
    ParsedFyPyFile_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files_map_by_key.abc_fy import (
    ParsedFyPyFilesMapByKey_PropertyMixin_ABC,
)
from fy_library.mixins.property.templates.template_model.abc_fy import (
    TemplateModel_PropertyMixin_ABC,
)


# fy:start ===>>>
class TemplateModel_UsingParsedFyPyFile_PropertyMixin(
    # Property Mixins
    AbstractEntitiesOrderingIndex_PropertyMixin_ABC,
    ParsedFyPyFile_PropertyMixin_ABC,
    ParsedFyPyFilesMapByKey_PropertyMixin_ABC,
    TemplateModel_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _template_model(self) -> BaseTemplateModel:
        # fy:end <<<===
        return CreateTemplateModelUsingParsedFyPyFile_Flow(
            parsed_fy_py_file=self._parsed_fy_py_file,
            parsed_fy_py_files_map_by_key=self._parsed_fy_py_files_map_by_key,
            abstract_entities_ordering_index=self._abstract_entities_ordering_index,
        )()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from typing import List


flow method_imports -> List[str]:
    property abstract_property_mixins using setter
    property abstract_method_mixins using setter
    property mixin_import_map using setter
    property parsed_method_fy_py_file using setter
    property import_generic using constant(["from typing import Generic"])
    property import_abc using when_abstract_property_and_abstract_method_exists
    property import_abstract_property_mixins using abstract_property_mixin_and_mixin_import_map
    property import_abstract_method_mixins using abstract_method_mixin_and_mixin_import_map
fy"""

from typing import Any
from typing import List

from fy_core.base.flow_base import FlowBase
from fy_library.domain.mixin_models import AbstractMethodModel, AbstractPropertyModel
from fy_library.domain.parsed_fy_py_file import ParsedMethodFyPyFile
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.mixins.property.entity_mixins.abstract_method_mixins.using_setter import (
    AbstractMethodMixins_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.entity_mixins.abstract_property_mixins.using_setter import (
    AbstractPropertyMixins_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.imports_and_user_imports.import_generic.using_setter import (
    ImportGeneric_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.imports_and_user_imports.imports.import__abstract_property_mixins__using_abstract_property_mixin_and_mixin_import_map__fy import (
    ImportAbstractPropertyMixins_UsingAbstractPropertyMixinAndMixinImportMap_PropertyMixin,
)
from fy_library.mixins.property.imports_and_user_imports.imports.import_abc__using_when_abstract_property_and_abstract_method_exists__fy import (
    ImportAbc_UsingWhenAbstractPropertyAndAbstractMethodExists_PropertyMixin,
)
from fy_library.mixins.property.imports_and_user_imports.imports.import_abstract_method_mixins__using_abstract_method_mixin_and_mixin_import_map__fy import (
    ImportAbstractMethodMixins_UsingAbstractMethodMixinAndMixinImportMap_PropertyMixin,
)
from fy_library.mixins.property.imports_and_user_imports.mixin_import_map.using_setter import (
    MixinImportMap_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.parsed_entity_fy_py_file.parsed_method_fy_py_file.using_setter import (
    ParsedMethodFyPyFile_UsingSetter_PropertyMixin,
)


# fy:start ===>>>
class MethodImports_Flow(
    # Property Mixins
    AbstractPropertyMixins_UsingSetter_PropertyMixin,
    AbstractMethodMixins_UsingSetter_PropertyMixin,
    MixinImportMap_UsingSetter_PropertyMixin,
    ParsedMethodFyPyFile_UsingSetter_PropertyMixin,
    ImportGeneric_UsingSetter_PropertyMixin,
    ImportAbc_UsingWhenAbstractPropertyAndAbstractMethodExists_PropertyMixin,
    ImportAbstractPropertyMixins_UsingAbstractPropertyMixinAndMixinImportMap_PropertyMixin,
    ImportAbstractMethodMixins_UsingAbstractMethodMixinAndMixinImportMap_PropertyMixin,
    # Base
    FlowBase[List[str]],
):
    def __init__(
        self,
        *args: Any,
        abstract_property_mixins: List[AbstractPropertyModel],
        abstract_method_mixins: List[AbstractMethodModel],
        mixin_import_map: dict[tuple[ParsedFyPyFileKind, str], str],
        parsed_method_fy_py_file: ParsedMethodFyPyFile,
        **kwargs: Any,
    ):
        self._abstract_property_mixins = abstract_property_mixins
        self._abstract_method_mixins = abstract_method_mixins
        self._mixin_import_map = mixin_import_map
        self._parsed_method_fy_py_file = parsed_method_fy_py_file
        self._import_generic = ["from typing import Generic"]
        super().__init__(*args, **kwargs)

    def __call__(self) -> List[str]:
        # fy:end <<<===
        method_generic_import = (
            self._import_generic
            if self._parsed_method_fy_py_file.generics_def != ""
            else []
        )

        return (
            self._import_abc
            + method_generic_import
            + self._import_abstract_method_mixins
            + self._import_abstract_property_mixins
        )



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from typing import List


property import_abstract_property_mixins: List[str] using abstract_property_mixin_and_mixin_import_map:
    property mixin_import_map
    property abstract_property_mixins
fy"""

import abc
from functools import cached_property
from typing import List

from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.mixins.property.entity_mixins.abstract_property_mixins.abc_fy import (
    AbstractPropertyMixins_PropertyMixin_ABC,
)
from fy_library.mixins.property.imports_and_user_imports.mixin_import_map.abc_fy import (
    MixinImportMap_PropertyMixin_ABC,
)


# fy:start ===>>>
class ImportAbstractPropertyMixins_UsingAbstractPropertyMixinAndMixinImportMap_PropertyMixin(
    # Property Mixins
    AbstractPropertyMixins_PropertyMixin_ABC,
    MixinImportMap_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _import_abstract_property_mixins(self) -> List[str]:
        # fy:end <<<===
        return [
            # property mixins
            self._mixin_import_map[
                (
                    ParsedFyPyFileKind(abstract_property_mixin.kind.value),
                    abstract_property_mixin.property_name.snake_case,
                )
            ]
            for abstract_property_mixin in self._abstract_property_mixins
        ]



# fy:start ===>>>
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind


class AbstractEntitiesOrderingIndex_UsingSetter_PropertyMixin:
    @property
    def _abstract_entities_ordering_index(
        self,
    ) -> dict[tuple[ParsedFyPyFileKind, str], int]:
        return self.__abstract_entities_ordering_index

    @_abstract_entities_ordering_index.setter
    def _abstract_entities_ordering_index(
        self,
        abstract_entities_ordering_index: dict[tuple[ParsedFyPyFileKind, str], int],
    ) -> None:
        self.__abstract_entities_ordering_index = abstract_entities_ordering_index


# fy:end <<<===
